using System;
using System.Collections;

namespace LambdaCalculus
{
	/// <summary>
	/// Identifier is a class that represents a variable within a Lambda Expression.
	/// </summary>
	internal class Identifier : Expression
	{
    /// <summary>
    /// Construct a new Identifier.
    /// </summary>
		internal Identifier()
		{
		}

    public override int IdentifierCount
    {
      get
      {
        return 1;
      }
    }


    internal override Expression Replace(Identifier oIdentifier, Expression oExpression)
    {
      if (this == oIdentifier)
        return oExpression.Copy();
      else
        return this;
    }

    public override Expression Evaluate()
    {
      return this;
    }

    internal override string ToString(IdentifierHash oIdentifiersInUse)
    {
      return String.Format( @"{0}", oIdentifiersInUse[this] );
    }

    public override Expression Copy()
    {
      return this;
    }

    public override int FunctionDepth
    {
      get
      {
        return 0;
      }
    }

    internal override bool SameAs(Expression oExpression, ArrayList oThisIdentifiers, ArrayList oExpressionIdentifiers)
    {
      return  oExpression.GetType() == typeof(Identifier) &&
        oThisIdentifiers.IndexOf(this) == oExpressionIdentifiers.IndexOf((Identifier) oExpression);
    }

    public override int LambdaCount
    {
      get
      {
        return 0;
      }
    }

    internal override string Abbreviate(IdentifierHash oIdentifiersInUse)
    {
      return this.ToString(oIdentifiersInUse);
    }

    internal override string GetAbbreviatedLatex(IdentifierHash oIdentifiersInUse)
    {
      return this.ToString(oIdentifiersInUse);
    }

    internal override string GetFullLatex(IdentifierHash oIdentifiersInUse)
    {
      return this.ToString(oIdentifiersInUse);
    }

	}
}
