using System;
using System.Collections;
using System.IO;
using System.Text.RegularExpressions;
using System.Windows.Forms;

using LambdaCalculus;

namespace LambdaCalculusGUI
{
	/// <summary>
	/// A class for the main application form.
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
    private System.Windows.Forms.Label lblExpression;
    private System.Windows.Forms.Button btnEvaluate;
    private System.Windows.Forms.Label label1;
    private System.Windows.Forms.TextBox txtResult;
    private System.Windows.Forms.TabPage tabDefinitions;
    private System.Windows.Forms.MainMenu mnuMain;
    private System.Windows.Forms.MenuItem mnuAbout;
    private System.Windows.Forms.MenuItem mnuOpen;
    private System.Windows.Forms.MenuItem mnuExit;
    private System.Windows.Forms.MenuItem menuItem3;
    private System.Windows.Forms.MenuItem menuItem5;
    private System.Windows.Forms.TextBox txtDefinitions;
    private System.Windows.Forms.OpenFileDialog diaOpenFileDialog;
    private System.Windows.Forms.MenuItem mnuFile;
    private System.Windows.Forms.MenuItem mnuHelp;
    private System.Windows.Forms.MenuItem mnuSaveAs;
    private System.Windows.Forms.SaveFileDialog diaSaveFileDialog;
    private System.Windows.Forms.TextBox txtExpressionToEvaluate;
    private System.Windows.Forms.MenuItem mnuFileOutputs;
    private System.Windows.Forms.TextBox txtFullResult;
    private System.Windows.Forms.TextBox txtAbbreviatedResult;
    private System.Windows.Forms.MenuItem mnuComplexities;
    private System.Windows.Forms.TabPage tabHelp;
    private System.Windows.Forms.MenuItem mnuHelpTab;
    private System.Windows.Forms.TabControl Tabs;
    private System.Windows.Forms.TextBox txtHelp;
    private System.Windows.Forms.MenuItem menuItem1;
    private System.Windows.Forms.TabPage tabResult;
    private System.Windows.Forms.TabPage tabResultWithNamedFunctions;
    private System.Windows.Forms.MenuItem mnuLatexWithNamedFunctions;
    private System.Windows.Forms.MenuItem mnuPlainText;
    private System.Windows.Forms.MenuItem mnuPlainTextWithNamedFunctions;
    private System.Windows.Forms.MenuItem mnuLatex;
    private System.ComponentModel.IContainer components;

    /// <summary>
    /// Construct the main form, initialize the text displayed on the help tab, get the execution directory, and hide
    /// the help tab.
    /// </summary>
		public MainForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

      txtHelp.Text =
"This an application which interprets untyped lambda calculus expressions written in plain text and evaluates them in normal order.\r\n" +
"\r\n" +
"To evaluate a lambda calculus expression, simply type it into the \"Expression\" field on the form and click the \"Evaluate\" button." +
"\r\n" +
"The plain text expressions this applications understands look like conventional lambda calculus expressions, but with the backslash character (\\) used to represent lambda and the period character (.) used to represent the dot.  All function arguments in the expressions must consist of single, lowercase letters.  For example,\r\n" +
"\t(\\x.(\\y.x))\r\n" +
"is a valid expression and\r\n" +
"\t(\\arg1.(\\arg2.arg1))\r\n" +
"is not.\r\n" +
"\r\n" +
"The application can handle extra parentheses, extra whitespace and functions with multiple arguments, hence\r\n" +
"\t(\\x.(\\y.x))\r\n" +
"\t\\xy.x\r\n" +
"\t(\\x y.x)\r\n" +
"\t(\\xy.(( x )))\r\n" +
"are all valid and equivalent expressions.\r\n" +
"\r\n" +
"There is no need for spaces in between function arguments where they are declared, but spaces or parentheses are required in between elements in function bodies, thus\r\n" +
"\t\\xyz.x y z\r\n" +
"\t\\xyz.(x)(y) z\r\n" +
"are valid, but\r\n" +
"\t\\xyz.xyz\r\n" +
"is not.\r\n" +
"\r\n" +
"Named functions can be defined on the \"Definitions\" tab.  Function names may not begin with numbers or lowercase letters and may not contain parentheses, semicolons, backslashes, periods, or pound signs (#).  A named function is declared like so:\r\n" +
"\tTrue := \\xy.x;\r\n" +
"\r\n" +
"Definitions may be spread across multiple lines.  A definition must end with a semicolon.\r\n" +
"\r\n" +
"A pound sign and any text following it on the same line is ignored by the interpreter.  Use this for providing comments on definitions:\r\n" +
"\tS := \\xyz.y(x y z); #successor function for natural numbers\r\n" +
"\r\n" +
"When the application reads a definition that uses the \":=\" operator, it will automatically try to normalize it before storing it.  Some functions cannot be normalized; they will expand indefinitely.  Declare these functions with the \".=\" operator to let Redboard know it should not attempt to normalize them.  For instance, the Y-combinator should be defined like so:\r\n" +
"\tY .= \\y.(\\x.y(x x))(\\x.y(x x));\r\n" +
"\r\n" +
"The application automatically interprets natural numbers (0,1,2,3,etc.) as Church numerals.\r\n" +
"\r\n" +
"Checking items in the \"File Outputs\" menu will cause the application to print output files to the directory it is running in.  The files will be created or superceded each time an expression is evaluated.  The files list results for each step in the evaluation.  Be aware that these file outputs can be quite large and take a lot of time to produce.\r\n";

      sExecutionDirectory = Directory.GetCurrentDirectory();
      Tabs.TabPages.Remove(tabHelp);
      Tabs.SelectedTab = tabDefinitions;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
      this.components = new System.ComponentModel.Container();
      System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager( typeof( MainForm ) );
      this.txtFullResult = new System.Windows.Forms.TextBox();
      this.lblExpression = new System.Windows.Forms.Label();
      this.txtDefinitions = new System.Windows.Forms.TextBox();
      this.btnEvaluate = new System.Windows.Forms.Button();
      this.txtAbbreviatedResult = new System.Windows.Forms.TextBox();
      this.Tabs = new System.Windows.Forms.TabControl();
      this.tabDefinitions = new System.Windows.Forms.TabPage();
      this.tabHelp = new System.Windows.Forms.TabPage();
      this.txtHelp = new System.Windows.Forms.TextBox();
      this.tabResult = new System.Windows.Forms.TabPage();
      this.tabResultWithNamedFunctions = new System.Windows.Forms.TabPage();
      this.txtResult = new System.Windows.Forms.TextBox();
      this.label1 = new System.Windows.Forms.Label();
      this.mnuMain = new System.Windows.Forms.MainMenu( this.components );
      this.mnuFile = new System.Windows.Forms.MenuItem();
      this.mnuOpen = new System.Windows.Forms.MenuItem();
      this.menuItem3 = new System.Windows.Forms.MenuItem();
      this.mnuSaveAs = new System.Windows.Forms.MenuItem();
      this.menuItem5 = new System.Windows.Forms.MenuItem();
      this.mnuExit = new System.Windows.Forms.MenuItem();
      this.mnuFileOutputs = new System.Windows.Forms.MenuItem();
      this.mnuPlainText = new System.Windows.Forms.MenuItem();
      this.mnuPlainTextWithNamedFunctions = new System.Windows.Forms.MenuItem();
      this.mnuLatex = new System.Windows.Forms.MenuItem();
      this.mnuLatexWithNamedFunctions = new System.Windows.Forms.MenuItem();
      this.mnuComplexities = new System.Windows.Forms.MenuItem();
      this.mnuHelp = new System.Windows.Forms.MenuItem();
      this.mnuHelpTab = new System.Windows.Forms.MenuItem();
      this.menuItem1 = new System.Windows.Forms.MenuItem();
      this.mnuAbout = new System.Windows.Forms.MenuItem();
      this.diaOpenFileDialog = new System.Windows.Forms.OpenFileDialog();
      this.diaSaveFileDialog = new System.Windows.Forms.SaveFileDialog();
      this.txtExpressionToEvaluate = new System.Windows.Forms.TextBox();
      this.Tabs.SuspendLayout();
      this.tabDefinitions.SuspendLayout();
      this.tabHelp.SuspendLayout();
      this.tabResult.SuspendLayout();
      this.tabResultWithNamedFunctions.SuspendLayout();
      this.SuspendLayout();
      // 
      // txtFullResult
      // 
      this.txtFullResult.Anchor = ( (System.Windows.Forms.AnchorStyles) ( ( ( ( System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom )
                  | System.Windows.Forms.AnchorStyles.Left )
                  | System.Windows.Forms.AnchorStyles.Right ) ) );
      this.txtFullResult.BackColor = System.Drawing.Color.SaddleBrown;
      this.txtFullResult.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
      this.txtFullResult.Font = new System.Drawing.Font( "Century", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte) ( 0 ) ) );
      this.txtFullResult.ForeColor = System.Drawing.Color.LightSalmon;
      this.txtFullResult.Location = new System.Drawing.Point( 0, 0 );
      this.txtFullResult.Multiline = true;
      this.txtFullResult.Name = "txtFullResult";
      this.txtFullResult.ScrollBars = System.Windows.Forms.ScrollBars.Both;
      this.txtFullResult.Size = new System.Drawing.Size( 784, 365 );
      this.txtFullResult.TabIndex = 0;
      // 
      // lblExpression
      // 
      this.lblExpression.Anchor = ( (System.Windows.Forms.AnchorStyles) ( ( System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left ) ) );
      this.lblExpression.BackColor = System.Drawing.Color.Transparent;
      this.lblExpression.Font = new System.Drawing.Font( "Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte) ( 0 ) ) );
      this.lblExpression.ForeColor = System.Drawing.Color.WhiteSmoke;
      this.lblExpression.Location = new System.Drawing.Point( 8, 400 );
      this.lblExpression.Name = "lblExpression";
      this.lblExpression.Size = new System.Drawing.Size( 72, 16 );
      this.lblExpression.TabIndex = 2;
      this.lblExpression.Text = "Expression:";
      this.lblExpression.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
      // 
      // txtDefinitions
      // 
      this.txtDefinitions.AcceptsReturn = true;
      this.txtDefinitions.Anchor = ( (System.Windows.Forms.AnchorStyles) ( ( ( ( System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom )
                  | System.Windows.Forms.AnchorStyles.Left )
                  | System.Windows.Forms.AnchorStyles.Right ) ) );
      this.txtDefinitions.BackColor = System.Drawing.Color.SaddleBrown;
      this.txtDefinitions.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
      this.txtDefinitions.Font = new System.Drawing.Font( "Century", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte) ( 0 ) ) );
      this.txtDefinitions.ForeColor = System.Drawing.Color.LightSalmon;
      this.txtDefinitions.Location = new System.Drawing.Point( 0, 0 );
      this.txtDefinitions.Multiline = true;
      this.txtDefinitions.Name = "txtDefinitions";
      this.txtDefinitions.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
      this.txtDefinitions.Size = new System.Drawing.Size( 784, 365 );
      this.txtDefinitions.TabIndex = 3;
      // 
      // btnEvaluate
      // 
      this.btnEvaluate.Anchor = ( (System.Windows.Forms.AnchorStyles) ( ( System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right ) ) );
      this.btnEvaluate.Font = new System.Drawing.Font( "Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte) ( 0 ) ) );
      this.btnEvaluate.ForeColor = System.Drawing.Color.WhiteSmoke;
      this.btnEvaluate.Location = new System.Drawing.Point( 704, 400 );
      this.btnEvaluate.Name = "btnEvaluate";
      this.btnEvaluate.Size = new System.Drawing.Size( 75, 23 );
      this.btnEvaluate.TabIndex = 4;
      this.btnEvaluate.Text = "Evaluate";
      this.btnEvaluate.Click += new System.EventHandler( this.btnEvaluate_Click );
      // 
      // txtAbbreviatedResult
      // 
      this.txtAbbreviatedResult.Anchor = ( (System.Windows.Forms.AnchorStyles) ( ( ( ( System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom )
                  | System.Windows.Forms.AnchorStyles.Left )
                  | System.Windows.Forms.AnchorStyles.Right ) ) );
      this.txtAbbreviatedResult.BackColor = System.Drawing.Color.SaddleBrown;
      this.txtAbbreviatedResult.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
      this.txtAbbreviatedResult.Font = new System.Drawing.Font( "Century", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte) ( 0 ) ) );
      this.txtAbbreviatedResult.ForeColor = System.Drawing.Color.LightSalmon;
      this.txtAbbreviatedResult.Location = new System.Drawing.Point( 0, 0 );
      this.txtAbbreviatedResult.Multiline = true;
      this.txtAbbreviatedResult.Name = "txtAbbreviatedResult";
      this.txtAbbreviatedResult.ScrollBars = System.Windows.Forms.ScrollBars.Both;
      this.txtAbbreviatedResult.Size = new System.Drawing.Size( 784, 365 );
      this.txtAbbreviatedResult.TabIndex = 6;
      // 
      // Tabs
      // 
      this.Tabs.Anchor = ( (System.Windows.Forms.AnchorStyles) ( ( ( ( System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom )
                  | System.Windows.Forms.AnchorStyles.Left )
                  | System.Windows.Forms.AnchorStyles.Right ) ) );
      this.Tabs.Controls.Add( this.tabDefinitions );
      this.Tabs.Controls.Add( this.tabHelp );
      this.Tabs.Controls.Add( this.tabResult );
      this.Tabs.Controls.Add( this.tabResultWithNamedFunctions );
      this.Tabs.Location = new System.Drawing.Point( 0, 0 );
      this.Tabs.Name = "Tabs";
      this.Tabs.SelectedIndex = 0;
      this.Tabs.Size = new System.Drawing.Size( 792, 392 );
      this.Tabs.TabIndex = 7;
      // 
      // tabDefinitions
      // 
      this.tabDefinitions.BackColor = System.Drawing.Color.Black;
      this.tabDefinitions.Controls.Add( this.txtDefinitions );
      this.tabDefinitions.Location = new System.Drawing.Point( 4, 22 );
      this.tabDefinitions.Name = "tabDefinitions";
      this.tabDefinitions.Size = new System.Drawing.Size( 784, 366 );
      this.tabDefinitions.TabIndex = 0;
      this.tabDefinitions.Text = "Definitions";
      // 
      // tabHelp
      // 
      this.tabHelp.BackColor = System.Drawing.Color.Black;
      this.tabHelp.Controls.Add( this.txtHelp );
      this.tabHelp.Location = new System.Drawing.Point( 4, 22 );
      this.tabHelp.Name = "tabHelp";
      this.tabHelp.Size = new System.Drawing.Size( 784, 366 );
      this.tabHelp.TabIndex = 4;
      this.tabHelp.Text = "Help";
      // 
      // txtHelp
      // 
      this.txtHelp.Anchor = ( (System.Windows.Forms.AnchorStyles) ( ( ( ( System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom )
                  | System.Windows.Forms.AnchorStyles.Left )
                  | System.Windows.Forms.AnchorStyles.Right ) ) );
      this.txtHelp.BackColor = System.Drawing.Color.SaddleBrown;
      this.txtHelp.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
      this.txtHelp.Font = new System.Drawing.Font( "Century", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte) ( 0 ) ) );
      this.txtHelp.ForeColor = System.Drawing.Color.LightSalmon;
      this.txtHelp.Location = new System.Drawing.Point( 0, -1 );
      this.txtHelp.Multiline = true;
      this.txtHelp.Name = "txtHelp";
      this.txtHelp.ReadOnly = true;
      this.txtHelp.ScrollBars = System.Windows.Forms.ScrollBars.Both;
      this.txtHelp.Size = new System.Drawing.Size( 784, 368 );
      this.txtHelp.TabIndex = 7;
      // 
      // tabResult
      // 
      this.tabResult.BackColor = System.Drawing.Color.Black;
      this.tabResult.Controls.Add( this.txtFullResult );
      this.tabResult.Location = new System.Drawing.Point( 4, 22 );
      this.tabResult.Name = "tabResult";
      this.tabResult.Size = new System.Drawing.Size( 784, 366 );
      this.tabResult.TabIndex = 3;
      this.tabResult.Text = "Result";
      // 
      // tabResultWithNamedFunctions
      // 
      this.tabResultWithNamedFunctions.BackColor = System.Drawing.Color.Black;
      this.tabResultWithNamedFunctions.Controls.Add( this.txtAbbreviatedResult );
      this.tabResultWithNamedFunctions.Location = new System.Drawing.Point( 4, 22 );
      this.tabResultWithNamedFunctions.Name = "tabResultWithNamedFunctions";
      this.tabResultWithNamedFunctions.Size = new System.Drawing.Size( 784, 366 );
      this.tabResultWithNamedFunctions.TabIndex = 2;
      this.tabResultWithNamedFunctions.Text = "Result with Named Functions";
      // 
      // txtResult
      // 
      this.txtResult.Anchor = ( (System.Windows.Forms.AnchorStyles) ( ( ( System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left )
                  | System.Windows.Forms.AnchorStyles.Right ) ) );
      this.txtResult.BackColor = System.Drawing.Color.SaddleBrown;
      this.txtResult.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
      this.txtResult.Font = new System.Drawing.Font( "Century", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte) ( 0 ) ) );
      this.txtResult.ForeColor = System.Drawing.Color.LightSalmon;
      this.txtResult.Location = new System.Drawing.Point( 88, 424 );
      this.txtResult.Name = "txtResult";
      this.txtResult.ReadOnly = true;
      this.txtResult.Size = new System.Drawing.Size( 608, 21 );
      this.txtResult.TabIndex = 8;
      // 
      // label1
      // 
      this.label1.Anchor = ( (System.Windows.Forms.AnchorStyles) ( ( System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left ) ) );
      this.label1.BackColor = System.Drawing.Color.Transparent;
      this.label1.Font = new System.Drawing.Font( "Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte) ( 0 ) ) );
      this.label1.ForeColor = System.Drawing.Color.WhiteSmoke;
      this.label1.Location = new System.Drawing.Point( 8, 424 );
      this.label1.Name = "label1";
      this.label1.Size = new System.Drawing.Size( 72, 16 );
      this.label1.TabIndex = 9;
      this.label1.Text = "Result:";
      this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
      // 
      // mnuMain
      // 
      this.mnuMain.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.mnuFile,
            this.mnuFileOutputs,
            this.mnuHelp} );
      // 
      // mnuFile
      // 
      this.mnuFile.Index = 0;
      this.mnuFile.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.mnuOpen,
            this.menuItem3,
            this.mnuSaveAs,
            this.menuItem5,
            this.mnuExit} );
      this.mnuFile.Text = "&File";
      // 
      // mnuOpen
      // 
      this.mnuOpen.Index = 0;
      this.mnuOpen.Text = "&Open...";
      this.mnuOpen.Click += new System.EventHandler( this.mnuOpen_Click );
      // 
      // menuItem3
      // 
      this.menuItem3.Enabled = false;
      this.menuItem3.Index = 1;
      this.menuItem3.Text = "&Save";
      this.menuItem3.Click += new System.EventHandler( this.mnuSave_Click );
      // 
      // mnuSaveAs
      // 
      this.mnuSaveAs.Index = 2;
      this.mnuSaveAs.Text = "Save &As...";
      this.mnuSaveAs.Click += new System.EventHandler( this.mnuSaveAs_Click );
      // 
      // menuItem5
      // 
      this.menuItem5.Index = 3;
      this.menuItem5.Text = "-";
      // 
      // mnuExit
      // 
      this.mnuExit.Index = 4;
      this.mnuExit.Text = "E&xit";
      this.mnuExit.Click += new System.EventHandler( this.mnuExit_Click );
      // 
      // mnuFileOutputs
      // 
      this.mnuFileOutputs.Index = 1;
      this.mnuFileOutputs.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.mnuPlainText,
            this.mnuPlainTextWithNamedFunctions,
            this.mnuLatex,
            this.mnuLatexWithNamedFunctions,
            this.mnuComplexities} );
      this.mnuFileOutputs.Text = "File &Outputs";
      // 
      // mnuPlainText
      // 
      this.mnuPlainText.Index = 0;
      this.mnuPlainText.Text = "&Plain Text";
      this.mnuPlainText.Click += new System.EventHandler( this.OnCheckableMenuItemClick );
      // 
      // mnuPlainTextWithNamedFunctions
      // 
      this.mnuPlainTextWithNamedFunctions.Index = 1;
      this.mnuPlainTextWithNamedFunctions.Text = "Plain Text with &Named Functions";
      this.mnuPlainTextWithNamedFunctions.Click += new System.EventHandler( this.OnCheckableMenuItemClick );
      // 
      // mnuLatex
      // 
      this.mnuLatex.Index = 2;
      this.mnuLatex.Text = "&LaTeX";
      this.mnuLatex.Click += new System.EventHandler( this.OnCheckableMenuItemClick );
      // 
      // mnuLatexWithNamedFunctions
      // 
      this.mnuLatexWithNamedFunctions.Index = 3;
      this.mnuLatexWithNamedFunctions.Text = "LaTeX with Named &Functions";
      this.mnuLatexWithNamedFunctions.Click += new System.EventHandler( this.OnCheckableMenuItemClick );
      // 
      // mnuComplexities
      // 
      this.mnuComplexities.Index = 4;
      this.mnuComplexities.Text = "&Complexities";
      this.mnuComplexities.Click += new System.EventHandler( this.OnCheckableMenuItemClick );
      // 
      // mnuHelp
      // 
      this.mnuHelp.Index = 2;
      this.mnuHelp.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.mnuHelpTab,
            this.menuItem1,
            this.mnuAbout} );
      this.mnuHelp.Text = "&Help";
      // 
      // mnuHelpTab
      // 
      this.mnuHelpTab.Index = 0;
      this.mnuHelpTab.Text = "Help &Tab";
      this.mnuHelpTab.Click += new System.EventHandler( this.mnuHelpTab_Click );
      // 
      // menuItem1
      // 
      this.menuItem1.Index = 1;
      this.menuItem1.Text = "-";
      // 
      // mnuAbout
      // 
      this.mnuAbout.Index = 2;
      this.mnuAbout.Text = "&About";
      this.mnuAbout.Click += new System.EventHandler( this.mnuAbout_Click );
      // 
      // diaOpenFileDialog
      // 
      this.diaOpenFileDialog.AddExtension = false;
      this.diaOpenFileDialog.Filter = "Lambda Expression Files (*.lmd)|*.lmd|All files (*.*)|*.*";
      this.diaOpenFileDialog.Title = "Open a Lambda Expression Definition File";
      // 
      // diaSaveFileDialog
      // 
      this.diaSaveFileDialog.DefaultExt = "lmd";
      this.diaSaveFileDialog.Filter = "Lambda Expression Files (*.lmd)|*.lmd|All files (*.*)|*.*";
      this.diaSaveFileDialog.Title = "Save Definitions File As...";
      // 
      // txtExpressionToEvaluate
      // 
      this.txtExpressionToEvaluate.Anchor = ( (System.Windows.Forms.AnchorStyles) ( ( ( System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left )
                  | System.Windows.Forms.AnchorStyles.Right ) ) );
      this.txtExpressionToEvaluate.BackColor = System.Drawing.Color.SaddleBrown;
      this.txtExpressionToEvaluate.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
      this.txtExpressionToEvaluate.Font = new System.Drawing.Font( "Century", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte) ( 0 ) ) );
      this.txtExpressionToEvaluate.ForeColor = System.Drawing.Color.LightSalmon;
      this.txtExpressionToEvaluate.Location = new System.Drawing.Point( 88, 400 );
      this.txtExpressionToEvaluate.Name = "txtExpressionToEvaluate";
      this.txtExpressionToEvaluate.Size = new System.Drawing.Size( 608, 21 );
      this.txtExpressionToEvaluate.TabIndex = 11;
      // 
      // MainForm
      // 
      this.AcceptButton = this.btnEvaluate;
      this.AutoScaleBaseSize = new System.Drawing.Size( 5, 13 );
      this.BackColor = System.Drawing.Color.Black;
      this.ClientSize = new System.Drawing.Size( 792, 449 );
      this.Controls.Add( this.txtExpressionToEvaluate );
      this.Controls.Add( this.txtResult );
      this.Controls.Add( this.label1 );
      this.Controls.Add( this.Tabs );
      this.Controls.Add( this.btnEvaluate );
      this.Controls.Add( this.lblExpression );
      this.ForeColor = System.Drawing.Color.Transparent;
      this.Icon = ( (System.Drawing.Icon) ( resources.GetObject( "$this.Icon" ) ) );
      this.Menu = this.mnuMain;
      this.Name = "MainForm";
      this.Text = "Lambda Calculus Evaluator";
      this.Tabs.ResumeLayout( false );
      this.tabDefinitions.ResumeLayout( false );
      this.tabDefinitions.PerformLayout();
      this.tabHelp.ResumeLayout( false );
      this.tabHelp.PerformLayout();
      this.tabResult.ResumeLayout( false );
      this.tabResult.PerformLayout();
      this.tabResultWithNamedFunctions.ResumeLayout( false );
      this.tabResultWithNamedFunctions.PerformLayout();
      this.ResumeLayout( false );
      this.PerformLayout();

    }
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			System.Windows.Forms.Application.Run(new MainForm());
		}

    #region Regular Expressions

    /// <summary>
    /// a regular expression representing code comments for multiline text strings
    /// </summary>
    private static Regex rComment = new Regex( @"#.*?$", RegexOptions.Multiline );
    /// <summary>
    /// a regular expression representing any combination of consecutive endline and carriage return characters
    /// </summary>
    private static Regex rEndlines = new Regex(@"[\n\r]+");
    /// <summary>
    /// a regular expression representing any combination of consecutive semicolons
    /// </summary>
    private static Regex rStatementDelimiters = new Regex(@";+");
    /// <summary>
    /// a regular expression representing any instance of non-whitespace characters
    /// </summary>
    private static Regex rNonWhitespace = new Regex(@"\S");

    #endregion

    #region Private Data Members and Private Properties

    /// <summary>
    /// a path for the directory the application is currently being run in.
    /// </summary>
    private string sExecutionDirectory;

    /// <summary>
    /// Get/set the last file name selected by the Save, Save As, or Open file dialogs. Display this file name
    /// with the application's title
    /// </summary>
    private string CurrentFileName
    {
      get
      {
        return diaSaveFileDialog.FileName;
      }
      set
      {
        diaOpenFileDialog.FileName = value;
        diaSaveFileDialog.FileName = value;
        this.Text = String.Format(@"Lambda Calculus Evaluator - ({0})", value );
        menuItem3.Enabled = true;
      }
    }

    /// <summary>
    /// Get/set the initial directory that the Open and Save file dialogs will go to.
    /// </summary>
    private string CurrentDirectory
    {
      get
      {
        return diaSaveFileDialog.InitialDirectory;
      }
      set
      {
        diaOpenFileDialog.InitialDirectory = value;
        diaSaveFileDialog.InitialDirectory = value;
      }
    }

    #endregion

    #region Event Handlers

    /// <summary>
    /// Respond to the "Evaluate" button being clicked.  Read and evaluate any definitions from the definitions tab,
    /// attempt to normalize the expression in the "Expression" text box and, if that succeeds, put the results
    /// on the results tab and then re-evaluate the expression and print the results to any of the output files
    /// selected in the menu structure.
    /// </summary>
    /// <param name="sender">the event sender</param>
    /// <param name="evarg">event arguments</param>
    private void btnEvaluate_Click(object sender, System.EventArgs evarg)
    {
      System.Windows.Forms.Cursor.Current = Cursors.WaitCursor;
      try
      {
        Expression.ClearDefinitions();
        Expression.Define(ParseDefinitions(txtDefinitions.Text));
        Expression oExpression = Expression.Parse(txtExpressionToEvaluate.Text);

        Stopwatch.Start();
        Expression oNormalizedExpression = oExpression.Normalize();
        TimeSpan oElapsedTime = Stopwatch.ElapsedTime;

        txtResult.Text = oNormalizedExpression.Abbreviation;
        txtFullResult.Lines = new string[]{ oExpression.ToString(),
                                            @"",
                                            @"# Evaluates to:",
                                            @"",
                                            oNormalizedExpression.ToString(),
                                            @"",
                                            String.Format( @"# Elapsed time: {0}", oElapsedTime ) };
        txtAbbreviatedResult.Lines = new string[]{ oExpression.Abbreviation,
                                                   @"",
                                                   @"# Evaluates to:",
                                                   @"",
                                                   txtResult.Text,
                                                   @"",
                                                   String.Format( @"# Elapsed time: {0}", oElapsedTime ) };

        if ( mnuPlainText.Checked ||
             mnuPlainTextWithNamedFunctions.Checked ||
             mnuLatex.Checked ||
             mnuLatexWithNamedFunctions.Checked ||
             mnuComplexities.Checked )
        {
          Expression[] oNormalizationSteps;
          oExpression.Normalize(out oNormalizationSteps);

          if (mnuPlainText.Checked)
            PrintPlainTextResults(oNormalizationSteps);
          if (mnuPlainTextWithNamedFunctions.Checked)
            PrintPlainTextResultsWithNamedFunctions(oNormalizationSteps);
          if (mnuLatex.Checked)
            PrintLatexResults(oNormalizationSteps);
          if (mnuLatexWithNamedFunctions.Checked)
            PrintLatexResultsWithNamedFunctions(oNormalizationSteps);
          if (mnuComplexities.Checked)
            PrintComplexities(oNormalizationSteps);
        }
      }
      catch (ApplicationException oEx)
      {
        txtResult.Text = "";
        MessageBox.Show(oEx.Message);
      }
      System.Windows.Forms.Cursor.Current = Cursors.Default;
    }


    /// <summary>
    /// Prompt the user to select a file and open the file if one is selected.
    /// </summary>
    /// <param name="sender">the event's sender</param>
    /// <param name="e">event arguments</param>
    private void mnuOpen_Click(object sender, System.EventArgs e)
    {
      if (diaOpenFileDialog.ShowDialog(this) == System.Windows.Forms.DialogResult.Cancel)
        return;

      System.IO.TextReader oReader = System.IO.File.OpenText(diaOpenFileDialog.FileName);
      txtDefinitions.Text = oReader.ReadToEnd();
      oReader.Close();

      CurrentDirectory = diaOpenFileDialog.InitialDirectory;
      CurrentFileName = diaOpenFileDialog.FileName;
    }

    /// <summary>
    /// Prompt the user to select a file name and save the current definitions display to it one is selected.
    /// </summary>
    /// <param name="sender">the event's sender</param>
    /// <param name="e">event arguments</param>
    private void mnuSaveAs_Click(object sender, System.EventArgs e)
    {
      if (diaSaveFileDialog.ShowDialog(this) == System.Windows.Forms.DialogResult.Cancel)
        return;

      CurrentFileName = diaSaveFileDialog.FileName;
      mnuSave_Click( sender, e );
    }

    /// <summary>
    /// Save the current definitions display to the currently selected definitions file.
    /// </summary>
    /// <param name="sender">the event's sender</param>
    /// <param name="e">event arguments</param>
    private void mnuSave_Click(object sender, System.EventArgs e)
    {
      System.IO.StreamWriter oWriter = new System.IO.StreamWriter(CurrentFileName);
      oWriter.Write(txtDefinitions.Text);
      oWriter.Close();
    }

    /// <summary>
    /// Toggle the "Checked" property of a menu item.
    /// </summary>
    /// <param name="sender">the menu item to be toggled</param>
    /// <param name="e">event arguments</param>
    private void OnCheckableMenuItemClick(object sender, System.EventArgs e)
    {
      System.Windows.Forms.MenuItem mnuItem = (System.Windows.Forms.MenuItem) sender;
      mnuItem.Checked = !mnuItem.Checked;
    }

    /// <summary>
    /// Show some info about the application when the user clicks the "About" menu option.
    /// </summary>
    /// <param name="sender">the event's sender</param>
    /// <param name="e">event arguments</param>
    private void mnuAbout_Click(object sender, System.EventArgs e)
    {
      MessageBox.Show( this,
        "An untyped Lambda Calculus expression interpreter and evaluator.\n\nVersion 1.0\nMarch 2007\n\ncreated by MacKenzie Cumings\n(mack@somerby.net)\n",
        "About",
        MessageBoxButtons.OK,
        MessageBoxIcon.Information );
    }

    /// <summary>
    /// Exit the application when the user clicks the "Exit" menu option.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void mnuExit_Click(object sender, System.EventArgs e)
    {
      Application.Exit();
    }

    /// <summary>
    /// Toggle showing the "Help" tab when the user clicks the "Help Tab" menu option.
    /// </summary>
    /// <param name="sender">the event's sender</param>
    /// <param name="e">event arguments</param>
    private void mnuHelpTab_Click(object sender, System.EventArgs e)
    {
      OnCheckableMenuItemClick( sender, e );
      if (mnuHelpTab.Checked && !Tabs.Contains(tabHelp))
        Tabs.TabPages.Add(tabHelp);
      else if (!mnuHelpTab.Checked && Tabs.Contains(tabHelp))
      {
        TabPage oSelectedTab = Tabs.SelectedTab;
        Tabs.TabPages.Remove(tabHelp);
        if (oSelectedTab == tabHelp)
          Tabs.SelectedTab = tabDefinitions;
        else
          Tabs.SelectedTab = oSelectedTab;
      }
    }

    #endregion

    #region Private Instance Methods

    /// <summary>
    /// Print the string versions of an array of expressions to "Results.txt".
    /// </summary>
    /// <param name="oExpressions">an array of expressions</param>
    private void PrintPlainTextResults(Expression[] oExpressions)
    {
      System.IO.StreamWriter oWriter = new System.IO.StreamWriter(Path.Combine( sExecutionDirectory, @"Results.txt" ));
      foreach (Expression oExpression in oExpressions)
      {
        oWriter.WriteLine(oExpression.ToString());
        oWriter.WriteLine(@"");
      }
      oWriter.Close();
    }

    /// <summary>
    /// Print the string versions of an array of expressions, abbreviated with the names of named functions
    /// where possible,  to "ResultsWithNamedFunctions.txt".
    /// </summary>
    /// <param name="oExpressions">an array of expressions</param>
    private void PrintPlainTextResultsWithNamedFunctions(Expression[] oExpressions)
    {
      System.IO.StreamWriter oWriter = new System.IO.StreamWriter(Path.Combine( sExecutionDirectory, @"ResultsWithNamedFunctions.txt" ));
      foreach (Expression oExpression in oExpressions)
      {
        oWriter.WriteLine(oExpression.Abbreviation);
        oWriter.WriteLine(@"");
      }
      oWriter.Close();
    }

    /// <summary>
    /// Print some statistic regarding the complexity of an array of expressions to "Complexities.txt".
    /// </summary>
    /// <param name="oExpressions">an array of expressions</param>
    private void PrintComplexities(Expression[] oExpressions)
    {
      System.IO.StreamWriter oWriter = new System.IO.StreamWriter(Path.Combine( sExecutionDirectory, @"Complexities.txt" ));
      foreach (Expression oExpression in oExpressions)
      {
        oWriter.WriteLine( "Identifiers:\t{0}\tLambdas:\t{1}\tFunction Depth:\t{2}   ",
                           oExpression.IdentifierCount, oExpression.LambdaCount, oExpression.FunctionDepth );
      }
      oWriter.Close();
    }

    /// <summary>
    /// Print LaTex representations of an array of expressions to a simple LaTeX document in "Results.tex".
    /// </summary>
    /// <param name="oExpressions">an array of expressions</param>
    private void PrintLatexResults(Expression[] oExpressions)
    {
      System.IO.StreamWriter oWriter = new System.IO.StreamWriter(Path.Combine( sExecutionDirectory, @"Results.tex" ));

      oWriter.WriteLine(@"\documentclass{article}");
      oWriter.WriteLine(@"\begin{document}");
      oWriter.WriteLine(@"\begin{eqnarray}");

      for (int i = 0; i < oExpressions.Length - 1; i++)
      {
        oWriter.WriteLine( @"{0} \\", oExpressions[i].FullLatex );
      }
      oWriter.WriteLine(oExpressions[oExpressions.Length - 1].FullLatex);

      oWriter.WriteLine(@"\end{eqnarray}");
      oWriter.WriteLine(@"\end{document}");

      oWriter.Close();
    }

    /// <summary>
    /// Print LaTex representations of an array of expressions, abbreviated with the names of named functions
    /// where possible, to a simple LaTeX document in "ResultsWithNamedFunctions.tex".
    /// </summary>
    /// <param name="oExpressions">an array of expressions</param>
    private void PrintLatexResultsWithNamedFunctions(Expression[] oExpressions)
    {
      System.IO.StreamWriter oWriter = new System.IO.StreamWriter(Path.Combine( sExecutionDirectory, @"ResultsWithNamedFunctions.tex" ));

      oWriter.WriteLine(@"\documentclass{article}");
      oWriter.WriteLine(@"\begin{document}");
      oWriter.WriteLine(@"\begin{eqnarray}");

      for (int i = 0; i < oExpressions.Length - 1; i++)
      {
        oWriter.WriteLine( @"{0} \\", oExpressions[i].LatexAbbreviation );
      }
      oWriter.WriteLine(oExpressions[oExpressions.Length - 1].LatexAbbreviation);

      oWriter.WriteLine(@"\end{eqnarray}");
      oWriter.WriteLine(@"\end{document}");

      oWriter.Close();
    }

    #endregion

    #region Private Static Methods

    /// <summary>
    /// Turn the contents of a .lmd file into an array of definition strings.  Remove comments.
    /// </summary>
    /// <param name="sLambdaCode">the contents of a .lmd file</param>
    /// <returns>an array of strings that Expression class can interpret as function definitions</returns>
    private static string[] ParseDefinitions(string sLambdaCode)
    {
      // Remove comments.
      sLambdaCode = rComment.Replace( sLambdaCode, @"" );
      // Replace endlines with spaces.
      sLambdaCode = rEndlines.Replace( sLambdaCode, @" " );
      // Split on statement delimiters.
      string[] sDefinitions = rStatementDelimiters.Split(sLambdaCode);
      // Filter out blank lines.
      ArrayList oDefinitions = new ArrayList();
      foreach (string sStatement in sDefinitions)
      {
        if (rNonWhitespace.IsMatch(sStatement))
          oDefinitions.Add(sStatement.Trim());
        
      }
      return (string[]) oDefinitions.ToArray(typeof(string));
    }

    #endregion
	}
}
